/*****************************************************
Chip type               : ATtiny2313
AVR Core Clock frequency: 1,000000 MHz
*****************************************************/
#include <tiny2313.h>

#define button0 PIND.3
#define button1 PIND.4
#define output PORTD.6

#define A PORTB.0
#define B PORTB.1
#define C PORTB.2
#define D PORTB.3
#define E PORTB.4
#define F PORTB.5
#define G PORTB.6
#define DP PORTB.7
#define A0 PORTD.0
#define A1 PORTD.1
#define A2 PORTD.2

unsigned char display_buffer[3]; 
volatile unsigned char display_counter;
volatile unsigned int counter_s, timer0;
eeprom unsigned int ee_counter_s[20], position;
volatile bit display_nc, counter_nc, con;
enum btn_states {b_prs, b_rls, b_hold} btn0, btn1;


void ports_init ()
{
PORTB=0x00;
DDRB=0b11111111;
PORTD=0b00011000;
DDRD=0b01100111;
}
void tcnt0_init()
{
  TCCR0 = 0;
  TCCR0 |= 1<<CS01; //1MHz/8   
  TIMSK = 0; 
  TIMSK |= 1<<TOIE0;
}

void convert_data ()
{
 char i;
 unsigned int cnt_tmp;
 
 cnt_tmp=counter_s; 
 for (i=0;i<3;i++)
 {
  display_buffer[i] = cnt_tmp%10; cnt_tmp /=10; 
 }

 if(!display_buffer[2])
 {
    display_buffer[2]=10;
    if(!display_buffer[1]) {display_buffer[1]=10;}
 }
}
void display_data ()
{
(display_counter>0)?(display_counter--):(display_counter = 2);
A0 = 0; A1 = 0; A2 = 0;
  switch (display_buffer[display_counter])
 {
  case 0: A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1: A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2: A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3: A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4: A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5: A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6: A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7: A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8: A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9: A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;
  case 10: A=1; B=1; C=1; D=1; E=1; F=1; G=1; break; 
  default: A=1; B=1; C=1; D=1; E=1; F=1; G=0; break;  
 }   
   
   switch (display_counter)
 {
  case 0: A0 = 1; /*if(d_point_position == 0) {DP = 0;} else{DP = 1;}*/ break;
  case 1: A1 = 1; /*if(d_point_position == 1) {DP = 0;} else{DP = 1;}*/ break;
  case 2: A2 = 1; /*if(d_point_position == 2) {DP = 0;} else{DP = 1;}*/ break;
 } 

}
void skan_keys ()
{
static unsigned char count0,count1,count2,count3;

if (btn0 == b_rls){count2 = 0;}
if (btn1 == b_rls){count3 = 0;}

if (!button0) {count0++;} else {count0 = 0; btn0 = b_rls;}
if (!button1) {count1++;} else {count1 = 0; btn1 = b_rls;}

if ((count0 > 250)&&(btn0 != b_hold)) {btn0 = b_prs; count0 = 0; count2++;}
if ((count1 > 250)&&(btn1 != b_hold)) {btn1 = b_prs; count1 = 0; count3++;} 

if ((count2 > 10)&&(btn0 == b_prs)) {btn0 = b_hold; count2 = 0;}
if ((count3 > 10)&&(btn1 == b_prs)) {btn1 = b_hold; count3 = 0;}
}
void write_eeprom (unsigned int eest)
{
#asm("cli")
 m:
 if((ee_counter_s[position]!=eest)||(ee_counter_s[position+1]!=eest))
 {ee_counter_s[position]=ee_counter_s[position+1]=eest;} 
 if (ee_counter_s[position]!=ee_counter_s[position+1])
 {if(position<19){position++; goto m;}else{position=0;}} 
#asm("sei")
}
unsigned int read_eeprom ()
{
if(ee_counter_s[position]!=ee_counter_s[position+1])
{ee_counter_s[position]=ee_counter_s[position+1];}
if(ee_counter_s[position]!=ee_counter_s[position+1]) 
{if(position<19){position++;ee_counter_s[position+1]=ee_counter_s[position];}}
return ee_counter_s[position];
}

void modes_operation ()
{
 static bit tr0=0,tr1=0,tr2=0;
 static enum modes {set, reset, stop, run} mode = stop; 
  
 if((btn0==b_hold)&&(btn1==b_hold)&&(!tr0)&&(mode != set)){mode = set;tr0=1;counter_s=ee_counter_s[position];}
 else if((btn0==b_prs)&&(btn1==b_prs)&&(mode!=set)){mode = reset;}
 else if((btn0==b_prs)&&(mode!=set)){mode = stop;}
 else if((btn1==b_prs)&&(mode!=set)){mode = run;}
 
 switch (mode)
 {
  case set: 
  { 
  output=0;
  con=0;
  DP = 0;
  timer0 = 5000;
  
  if(btn0==b_rls){tr0=0;tr1=1;}
  if(btn1==b_rls){tr0=0;tr2=1;}   
  if((btn0==b_prs)&&(tr1)){tr1=0; timer0 = 5000; if(counter_s<999){counter_s++;}else{counter_s=0;}}
  if((btn1==b_prs)&&(tr2)){tr2=0; timer0 = 5000; if(counter_s){counter_s--;}else{counter_s=999;}}
  if((btn0==b_hold)&&(btn1!=b_hold)&&(counter_nc))
  {timer0 = 5000;counter_nc=0; if(counter_s<989){counter_s++;}else{counter_s=0;}}
  if((btn1==b_hold)&&(btn0!=b_hold)&&(counter_nc))
  {timer0 = 5000;counter_nc=0; if(counter_s){counter_s--;}else{counter_s=999;}}
  if((!timer0)||((btn0==b_hold)&&(btn1==b_hold)&&(!tr0))) 
  {write_eeprom(counter_s); mode=stop; btn0=b_rls; btn1=b_rls; DP = 1;} 
  if (display_nc){display_nc=0;convert_data();} 
  break;
  }
  case reset: 
  {
  output=0;
  con=0;
  counter_s=read_eeprom();
  if (display_nc){display_nc=0;convert_data();}  
  break;
  }
  case stop: 
  {
  output=0;
  con=0;
  if (display_nc){display_nc=0;convert_data();}
  break;
  }
  case run:
  {
  con=1; 
  if(counter_s){output=1;}else{output=0;}
  if (display_nc){display_nc=0;convert_data();}
  break;
  }
  default: break;
 }
}

interrupt [TIM0_OVF] void t0_ovf_isr()
{
static unsigned char n1;
static unsigned int counter_hs;
TCNT0=0x5;

if (counter_hs){counter_hs--;} 
else {counter_hs=500; display_nc=1; if((counter_s)&&(con)){counter_s--;}}
if(n1){n1--;} else{n1=50; counter_nc=1;}

if(timer0){timer0--;}
}

void main(void)
{
 ports_init();  
 tcnt0_init();
 con=0; 
 output = 0; 
if(ee_counter_s[0]==0xff){ee_counter_s[0]=100;}
if(ee_counter_s[1]!=ee_counter_s[0]){ee_counter_s[1]=ee_counter_s[0];}
if(position==0xff){position=0;}
counter_s = ee_counter_s[position];
 DP=1;
 UCR=0;
 #asm ("sei");
                                                                                         
while (1)
      {   
      display_data();
      skan_keys ();
      modes_operation ();
      }
}
